/*
 *	dlm2html.c		html converter for DoCoMo DataLinkSoftware(D208)
 *										Copyright(C)2000 By Kashi.
 *		2000/02/09	first edition
 *		2000/02/14	second edition		added '-l' option
 *		2000/02/16	#/b flag fixed
 */
#include <stdio.h>
#include <ctype.h>
#include "dlm.h"

#define	VERSION	"1.0.1"

static unsigned char	tonechar[7][5] = {
	"h", "", "~", "̧", "\", "", "V"
};
static int	rtempo[8][2] = {
	{0x12, 84}, {0x10, 94}, {0x0e, 108}, {0x0c, 126}, {0x0a, 150}, {0x08, 188}
};
static int	sharps_bt[8] = { 0, 1, 3, 4, 5, 7, 7, 7 };
static int	flats_bt[8] = { 1, 2, 4, 5, 6, 7, 7, 7 };
main( argc, argv)
int	argc;
char	**argv;
{
	FILE	*fp;
	struct	dlm	p2;
	int		i, j;
	int		s, tempo, tone, maxsize;
	int		offs, listmode;
	int		flats[8], sharps[8];

	if( argc != 3 ){
usage:
		fprintf( stderr, "usage: dlm2html [song_No|-l] dlm_file\n");
		exit(1);
	}

	listmode = 0; s = 1;
	if(!strcmp( argv[1], "-l")) listmode = 1;
	else if(isdigit(argv[1][0])) s = atoi( argv[1] );

	if( s < 0 || s > 19 ){
		fprintf( stderr, "dlm2html: song number is 0 to 19\n");
		exit(1);
	}
	if((fp = fopen( argv[2], "r")) == NULL){
		perror("dlm2html");
		exit(1);
	}
	if( fread( &p2, sizeof(p2), 1, fp) == (int)NULL ){
invstrc:
		fprintf( stderr, "dlm2html: invalid data structure\n");
		fclose(fp);
		exit(1);
	}
	if( strcmp( p2.header, "ްݸ")) goto invstrc;

	if( listmode ){
		for(i = 0; i < 20; i++){
#ifdef	hpux
			maxsize = ((p2.melodies[i].maxsize << 8) + (p2.melodies[i].maxsize >> 8)) & 0x00ff;
#else
			maxsize = p2.melodies[i].maxsize;
#endif
			printf("%2d. %-30s %3d steps\n", i, p2.melodies[i].name, maxsize);
		}
		exit(0);
	}


#ifdef	hpux
	maxsize = ((p2.melodies[s].maxsize << 8) + (p2.melodies[s].maxsize >> 8)) & 0x00ff;
#else
	maxsize = p2.melodies[s].maxsize;
#endif

	for(i = 0; i < 8; i++){ sharps[i] = 0; flats[i] = 0; }
	for(i = 0; i < 8; i++){
		if( p2.melodies[s].sharps & (1 << i) )
			sharps[sharps_bt[i]] = 1;
		if( p2.melodies[s].flats & (1 << i) )
			flats[flats_bt[i]] = 1;
	}
	printf("<h3>Ȗ: %30s</h3>\n", p2.melodies[s].name);
	printf("<h3>e|: ");

	tempo = -1;
	for( i = 0; i < 6; i++){
		if( rtempo[i][0] == p2.melodies[s].tempo){
			tempo = i + 1;;
			break;
		}
	}
	printf("%d (%d BPM)</h3>\n", tempo, rtempo[tempo - 1][1]);

	printf("<h3>F: ");
	switch( p2.melodies[s].toneflag1 + p2.melodies[s].toneflag2){
	case 0:
		printf("`");
		break;
	case 1:
		printf("a");
		break;
	case 2:
		printf("b");
		break;
	default:
		printf("H");
		break;
	}
	printf("</h3>\n");

	for(i = 0; i < 3; i++){
		printf("<br><br>\n");
		printf("<h3>p[g %d:</h3><dl><dl>\n", i + 1);
		printf("<h3>: ");
		switch( p2.melodies[s].vol[i] ){
		case 1:
			printf("` ()");
			break;
		case 2:
			printf("a ()");
			break;
		case 3:
			printf("b ()");
			break;
		default:
			printf("H");
			break;
		}
		printf("</h3>\n");

		printf("<table border=0 cellspacing=0 cellpadding=0>\n");
		for(j = 0; j < maxsize * 2; j += 16){
			printf("<tr>");
			for( offs = 0; offs < 16; offs += 2){
				if( offs + j > maxsize * 2) break;
				if( p2.melodies[s].datas[i][j + offs] == 0xff) break;
				printf("<th rowspan=2 width=32 valign=bottom align=right><font size=5>");
				if( p2.melodies[s].datas[i][j + offs + 1] == 0x80
				 || p2.melodies[s].datas[i][j + offs + 1] == 0xc0 ){
					if( p2.melodies[s].datas[i][j + offs] ){
						printf("%s", tonechar[(p2.melodies[s].datas[i][j+offs] - 1)& 7]);
						printf("</font></th><th width=16 valign=bottom align=left><font size=3>%d</font></th>",
								(p2.melodies[s].datas[i][j + offs] >> 5) + 1);
					} else {
						printf("");
						printf("</font></th><th width=16><font size=3>&nbsp;</font></th>");
					}
				} else {
					printf("-");
					printf("</font></th><th width=16><font size=3>&nbsp;</font></th>");
				}
			}
			printf("</tr>\n<tr>");
			for( offs = 0; offs < 16; offs += 2){
				if( offs + j > maxsize * 2) break;
				if( p2.melodies[s].datas[i][j + offs] == 0xff) break;
				printf("<th width=16 align=left><font size=3>");
				if( p2.melodies[s].datas[i][j + offs]
				  && (p2.melodies[s].datas[i][j + offs + 1] == 0x80
				   || p2.melodies[s].datas[i][j + offs + 1] == 0xc0) ){
					if( (p2.melodies[s].datas[i][j+offs] & 8 )
					 || sharps[(p2.melodies[s].datas[i][j+offs] & 7) - 1] ){
						printf("#");
					} else if( (p2.melodies[s].datas[i][j+offs] & 0x10 )
					 || flats[(p2.melodies[s].datas[i][j+offs] & 7) - 1] ){
						printf("b");
					} else goto noshift;
				} else {
noshift:
					printf("&nbsp;");
				}
				printf("</font></th>\n");
			}
			printf("</tr>\n<tr>");
			for( offs = 0; offs < 16; offs += 2){
				if( offs + j > maxsize * 2) break;
				if( p2.melodies[s].datas[i][j + offs] == 0xff) break;
				printf("<th colspan=2 width=48 valign=top><font size=2>");
				if( p2.melodies[s].datas[i][j + offs + 1] == 0x40 ){
					printf("<hr width=40 size=4 noshade align=left>");
				} else if(p2.melodies[s].datas[i][j + offs + 1] == 0x80 ){
					printf("<hr width=40 size=4 noshade align=right>");
				} else if( p2.melodies[s].datas[i][j + offs + 1] == 0xc0 ){
					printf("<hr width=32 size=4 noshade align=center>");
				} else {
					printf("<hr width=48 size=4 noshade align=center>");
				}
				printf("%03d</font><br><br></th>\n", (j + offs) / 2 + 1);
			}
		}
		printf("</table></dl></dl>\n");
	}
	printf("<div align=right>dlm2html Version %s<br>Copyright(C)2000 Kashi.</div>\n", VERSION);
	fclose(fp);
}

